#https://fedorahosted.org/sssd/wiki/BugLifecycle

[[PageOutline]]

== Bug Lifecycle for the System Security Services Daemon ==

=== Filing a new bug ===

Some bugs will be automatically cloned from distribution bug-trackers, such as http://bugzilla.redhat.com. People working with upstream code should file bugs directly in Trac.

==== Information to include ====
* Type
 Is this a defect or an enhancement? (Tasks should be reserved for internal development tasks, such as creating unit tests)
* short summary
 A brief description of the problem.[[br]]
 Good: "The LDAP provider segfaults when an invalid TLS certificate is specified"[[br]]
 Bad: "SSSD crashed"
* Full description
 A much more detailed description of the problem.[[br]]
 Good:
{{{
If you specify a path to the TLS certificate that does not exist, the LDAP provider crashes on startup. /var/log/messages reports that a segmentation fault occurred.
}}}
 Bad: Restate the short summary.
* Component
 Please select the SSSD component most accurate for your issue. This will ensure that the default assignee is set correctly.
* Version
 Please select the version of the SSSD that the bug occurs in. If the bug is in the current working tree, select the most recent released version.
* CC (optional)
 If there are other parties interested in a particular bug, please include their Fedora usernames here.

==== Information to leave alone ====
These options should be modified only by the SSSD developers and should not be changed from their defaults by the bug reporter.

* Priority
 This will be determined at weekly bug triage meetings
* Milestone
 This will be determined at weekly bug triage meetings
* Assign to
 This will be determined at weekly bug triage meetings
* Keywords
 These are used for very specific reasons and should generally be left blank

=== What to do when a bug is assigned to you ===
 1. Verify that the component and version is set correctly.
 1. Verify that you are the correct person to work on this bug, or reassign it accordingly.
 1. Triage the bug. See if you can reproduce it with the information given, or request more information from the submitter.
 1. Once the bug is triaged, adjust the priority of the bug as related to its severity. Use the following recommended guide for priority:
||'''Priority'''||'''Meaning'''||
||Blocker||Something is broken that prevents one or more major components from functioning at all||
||Critical||A serious, reproducable bug is found.||
||Major||There is a bug in the code, and it should be fixed||
||Minor||There is a bug in the code, and it should be fixed if there's spare time||
||Trivial||There's a typo or minor translation mistake||

 1. Discussion on IRC or sssd-devel@lists.fedorahosted.org will determine the appropriate milestone target for the fix.


=== What to do when you're working on a bug ===
 1. Accept the ticket. This marks its state as "Assigned" to indicate that you are working on it.
 1. Identify a fix and generate a patch for the fix.
{{{
git add <path_to_file1> <path_to_file2> ...
git commit
git format-patch -M -C --patience --full-index -1
}}}
 1. Send a code-review email to sssd-devel@lists.fedorahosted.org with the patch attached.
 1. The patch will be acked or nacked on the list. Repeat above until acked.

Note: if you want to send a series of patches, generate them with {{{git format-patch}}} as above, but change {{{-1}}} to {{{-N}}} (where N is the number of patches in the set).

=== What to do when you've fixed a bug ===
Once a patch has been acked and pushed to the upstream master, the following changes should be made to the bug.
 * The ticket should be marked "Resolve as fixed"
 * The "!FixedIn" field should be set to the UPCOMING release (not the current release). So if you're working on a bug in 0.4.1 and you fix it, it should be marked !FixedIn: 0.5.0
 * If the patch for this bug changed the expected output of an operation against the SSSD, mark the "Affects Tests" checkbox.
 * If the patch for this bug added, removed or changed a configuration option, mark both the "Affects Tests" and "Affects Doc" checkboxes.
 * The comment field for the resolution should include "Fixed in [<git commit id>]", as Trac will automatically create a link to the changelist.

=== What to do if a bug is not a bug ===
 * If a bug is a duplicate, resolve it as "duplicate" and include a comment stating the bug number of the original issue.
 * If a bug cannot be reproduced, resolve it as "worksforme"
 * If a bug was due to a misconfiguration, resolve it as "invalid"
 * If a bug is in a component that will no longer exist as of the next release (for example, a rewrite of a backend), resolve it as "wontfix"

