#https://fedorahosted.org/sssd/wiki/DevelTutorials

[[PageOutline]]

This page is a work-in-progress. Help us improve it! Submit your own tutorials below.

If you have questions, need help or want to discuss your ideas, please either send us an email at sssd-devel@lists.fedorahosted.org or join us on IRC in the #sssd channel on irc.freenode.net.

== Talloc ==
SSSD relies on the talloc memory allocator that originated in the Samba project.

SSSD developer Pavel Březina wrote a very detailed tutorial on its design, use and best practices. This is a must-read for anyone interested in contributing to SSSD.
 * [http://talloc.samba.org/talloc/doc/html/libtalloc__tutorial.html Talloc Tutorial]

Other resources:
 * [http://sgallagh.wordpress.com/2010/03/17/why-you-should-use-talloc-for-your-next-project/ Why you should use talloc for your next project]

== Tevent and tevent_req ==
 * We need a good tutorial on tevent_req, we don't have one yet.

== Coding Style ==
We have adopted the code style and formatting specification used by the FreeIPA project to describe our [http://www.freeipa.org/page/Coding_Style C] and [http://www.freeipa.org/page/Python_Coding_Style Python] coding styles.

== Code Contributions ==

=== Getting the source ===
SSSD uses the "git" source-control system to maintain its code. You can find an excellent tutorial for using git [http://schacon.github.com/git/gittutorial.html here].

To check out the latest SSSD sources, install git on your system (Instructions are for Fedora, you may need to use a platform-specific installation mechanism for other OSes).
{{{
yum install git
}}}

The first time, you will want to configure git with your name and email address for patch attribution:
{{{
git config --global user.name "GIVENNAME SURNAME"
git config --global user.email "username@domain.com"
}}}

Two other options you probably want to have set globally are:
{{{
git config --global color.ui auto
}}}
which enables syntax highlighting at the console for git commands such as 'git status' and 'git diff' and
{{{
yum install meld
git config --global merge.tool meld
}}}
which gives you a graphical tool for resolving merge conflicts.

Once that's done, you can clone our upstream git repository with
{{{
git clone git://git.fedorahosted.org/git/sssd.git
}}}

This will create a new subdirectory 'sssd' in the current directory which will contain the current master sources. All development should be done first against the master branch, then backported to one of the stable branches if necessary.

=== Building SSSD for development and debugging ===
Starting with SSSD 1.10 beta, we now include a set of helper aliases and environment variables in SSSD to simplify building development versions of SSSD on Fedora. To take advantage of them, use the following command:
{{{
. /path/to/sssd-source/contrib/fedora/bashrc_sssd
}}}

To build SSSD from source, follow these steps:

1. Install necessary tools
{{{
# when using yum
sudo yum install -y rpm-build yum-utils

# when using dnf
sudo dnf install -y rpm-build dnf-plugins-core
}}}

2. Create source rpm (run from git root directory)
{{{
contrib/fedora/make_srpm.sh
}}}

3. Install SSSD dependencies
{{{
# when using yum
sudo yum-builddep rpmbuild/SRPMS/sssd-*.src.rpm

# when using dnf
sudo dnf builddep rpmbuild/SRPMS/sssd-*.src.rpm
}}}

If you plan on working with the SSSD source often, you may want to add the following to your {{{~/.bashrc}}} file:
{{{
if [ -f /path/to/sssd-source/contrib/fedora/bashrc_sssd ]; then
    . /path/to/sssd-source/contrib/fedora/bashrc_sssd
fi
}}}

You can now produce a Debug build of SSSD by running:
{{{
cd /path/to/sssd-source
reconfig && chmake
}}}

The {{{reconfig}}} alias will run {{{autoreconf -if}}}, create a parallel build directory named after your CPU architecture (e.g. x86_64) and then run the configure script with the appropriate options for installing on a Fedora system with all experimental features enabled.

The {{{chmake}}} alias will then perform the build (with build messages suppressed except on warning or error) as well as build and run tests.

Finally, assuming you have sudo privilege, you can run the {{{sssinstall}}} alias to install this debug build onto the build machine.

To install SSSD on other distributions, you can run the usual Autotools combo:
{{{
autoreconf -i -f && \
./configure --enable-nsslibdir=/lib64 --enable-pammoddir=/lib64/security && \
make
sudo make install
}}}

To build and install the code. Please note that by default, the Autotools install prefix is `/usr/local`. Also, if you are building and installing on a 32bit machine, you should use `/lib/` instead of `/lib64` for `nsslibdir` and `pammoddir`. Please note that even if you are installing to `/usr/local`, the NSS and PAM libraries must be installed to system library directories as that's where NSS and PAM would be looking for them.

=== Unit tests ===
It is preferred if a patch comes together with a unit test. SSSD uses the [http://cmocka.org/ cmocka] library for developing unit tests, although some older tests still use the [http://check.sourceforge.net/ check framework]. For an introduction to cmocka, see either the examples in the `src/tests/cmocka` directory or read the [https://lwn.net/Articles/558106/ lwn article on cmocka].

=== Submitting a patch upstream ===
Make your changes and then add any new or modified files to a change-set with the command:
{{{
git add <path_to_file1> <path_to_file2> ...
git commit
}}}

Before submitting a patch, always make sure it applies to the latest upstream master branch. You will want to rebase to this branch and fix any merge conflicts (in case someone else changed the same code).
{{{
git remote update
git rebase -i origin/master
}}}

If this rebase has a merge conflict, you will need to resolve the conflict before you continue. If you get stuck or make a mistake, you can use
{{{
git rebase --abort
}}}
This will put you right back where you started.

Once your changes are ready for submission, you can use the following command to generate a patch
{{{
git format-patch -M -C --patience --full-index -1
}}}

This will make a local commit (it doesn't get pushed upstream) and then creates a patch locally. You can create multiple commits and then change {{{-1}}} to {{{-N}}} where N is the number of commits to produce patches for. To submit your patch for code-review and inclusion upstream, mail it as an attachment to {{{sssd-devel@lists.fedorahosted.org}}}.

If a patch isn't accepted on the first review, you will need to modify it and resubmit it. When this happens, you will want to amend your changes to the existing patch with
{{{
git add <files>
git commit --amend
}}}

If you need to make changes to earlier patches in your tree, you can use
{{{
git rebase -i origin/master
}}}
and follow the directions in the interactive rebase to modify specific patches.

=== Patch metadata ===
The description associated with the patch is an important piece of information that allows other developers or users to see what the change was about, what bug did the commit fix or what feature did the commit implement. To structure the information many SSSD developers use the following format:

* one-line short description
* blank line
* ticket or bugzilla URL (if available)
* blank line
* One or more paragraphs that describe the change if it can't be described fully in the one-line description

These best practices are loosely based on the [http://www.kernel.org/doc/Documentation/SubmittingPatches kernel patch submission recommendation].

An example of a patch formatted according to the above guidelines is commit e4d1e192430f6c76eee5b64decc3a799a0926979:
{{{
krb: recreate ccache if it was deleted

https://fedorahosted.org/sssd/ticket/1512

If directory where a ccache file was stored was missing and user
was still logged in, we erroneously considered the ccache file
still active. Thus the ccache file was not recreated and user was
unable to login.
}}}

== Translation Contributions ==
All SSSD translation efforts are maintained at [https://www.transifex.net/projects/p/sssd/ Transifex's SSSD Project]

== Devel Tips ==
There is a dedicated wiki page with assorted collection of [DevelTips tips for developers]

